@@ -29,7 +29,7 @@ pdl_error pdl_ ## name(pdl *a) { \
 		return pdl_make_error_simple(PDL_EUSERERROR, "pdl_" #name " without vaffine"); \
 	} \
 	PDL_ENSURE_ALLOCATED(a); \
-    PDL_GENERICSWITCH(intype, X, return pdl_make_error(PDL_EUSERERROR, "Not a known data type code=%d", intype)) \
+    PDL_GENERICSWITCH(PDL_TYPELIST2_ALL, intype, X, return pdl_make_error(PDL_EUSERERROR, "Not a known data type code=%d", intype)) \
     return PDL_err; \
 }
 
@@ -41,22 +41,13 @@ VAFF_IO(writebackdata_vaffine, X)
 #undef X
 #undef XCODE
 
-/* Various conversion utilities for pdl data types */
-
-/* Change the type of all the data in a pdl struct, either changing the
-   original perl structure or making a temporary copy  */
-
-/* 
- * it seems this does not have to be aware of bad values
- * (at least in the current scheme)
- */
-
 pdl_error pdl_converttype( pdl* a, int targtype ) {
     pdl_error PDL_err = {0, NULL, 0};
     void* b;     /* Scratch data ptr */
-    PDL_Indx   i;
     PDLDEBUG_f(printf("pdl_converttype %p, %d, %d\n", (void*)a, a->datatype,
-	targtype);)
+	targtype));
+    if(a->state & PDL_DONTTOUCHDATA)
+      return pdl_make_error_simple(PDL_EUSERERROR, "Trying to converttype magical (mmaped?) pdl");
 
     int intype = a->datatype;
     if (intype == targtype)
@@ -66,9 +57,6 @@ pdl_error pdl_converttype( pdl* a, int targtype ) {
 
     STRLEN nbytes = a->nvals * pdl_howbig(targtype); /* Size of converted data */
 
-    if(a->state & PDL_DONTTOUCHDATA) {
-      return pdl_make_error_simple(PDL_EUSERERROR, "Trying to convert of magical (mmaped?) pdl");
-    }
     if (diffsize) {
        b = a->data;                      /* pointer to old data */
        a->data     = pdl_smalloc(nbytes); /* Space for changed data */
@@ -77,19 +65,34 @@ pdl_error pdl_converttype( pdl* a, int targtype ) {
        b = a->data; /* In place */
     }
 
-    /* Do the conversion as nested switch statements */
-#define X_OUTER(datatype_out, ctype_out, ppsym_out, shortctype_out, defbval_out) \
-    ctype_out *bb = (ctype_out *) b; \
-    i = a->nvals; \
-    PDL_GENERICSWITCH2(targtype, X_INNER, return pdl_make_error(PDL_EUSERERROR, "Not a known data type code=%d", targtype))
-#define X_INNER(datatype_in, ctype_in, ppsym_in, shortctype_in, defbval_in) \
-    ctype_in *aa = (ctype_in *) a->data; \
+#define THIS_ISBAD(from_badval_isnan, from_badval, from_val) \
+  ((from_badval_isnan) \
+    ? isnan((double)(from_val)) \
+    : (from_val) == (from_badval))
+#define X_OUTER(datatype_from, ctype_from, ppsym_from, shortctype_from, defbval_from) \
+    PDL_Indx i = a->nvals; \
+    ctype_from *bb = (ctype_from *) b; \
+    ctype_from from_badval = pdl_get_pdl_badvalue(a).value.ppsym_from; \
+    char from_badval_isnan = PDL_ISNAN_##ppsym_from(from_badval); \
+    PDL_GENERICSWITCH2(PDL_TYPELIST2_ALL_, targtype, X_INNER, return pdl_make_error(PDL_EUSERERROR, "Not a known data type code=%d", targtype))
+#define X_INNER(datatype_to, ctype_to, ppsym_to, shortctype_to, defbval_to) \
+    ctype_to *aa = (ctype_to *) a->data; \
     aa += i-1; bb += i-1; \
-    while (i--) \
-      *aa-- = (ctype_in) *bb--;
-    PDL_GENERICSWITCH(intype, X_OUTER, return pdl_make_error(PDL_EUSERERROR, "Not a known data type code=%d", intype))
+    if (a->state & PDL_BADVAL) { \
+      ctype_to to_badval = defbval_to; \
+      a->has_badvalue = 0; \
+      while (i--) { \
+        *aa-- = THIS_ISBAD(from_badval_isnan, from_badval, *bb) \
+          ? to_badval : (ctype_to) *bb; \
+        bb--; \
+      } \
+    } else \
+      while (i--) \
+        *aa-- = (ctype_to) *bb--;
+    PDL_GENERICSWITCH(PDL_TYPELIST2_ALL, intype, X_OUTER, return pdl_make_error(PDL_EUSERERROR, "Not a known data type code=%d", intype))
 #undef X_INNER
 #undef X_OUTER
+#undef THIS_ISBAD
 
     /* Store new data */
     if (diffsize) {
