@@ -30,7 +30,7 @@ void pdl_SetSV_PDL ( SV *sv, pdl *it ) {
 size_t pdl_howbig (int datatype) {
 #define X(datatype, ctype, ppsym, shortctype, defbval) \
     return sizeof(ctype);
-  PDL_GENERICSWITCH(datatype, X, croak("Not a known data type code=%d", datatype))
+  PDL_GENERICSWITCH(PDL_TYPELIST2_ALL, datatype, X, croak("Not a known data type code=%d", datatype))
 #undef X
 }
 
@@ -38,9 +38,11 @@ size_t pdl_howbig (int datatype) {
 pdl_error pdl_makescratchhash(pdl *ret, PDL_Anyval data) {
   pdl_error PDL_err = {0, NULL, 0};
   STRLEN n_a;
+  size_t datasize = pdl_howbig(ret->datatype = data.type);
+  PDLDEBUG_f(printf("pdl_makescratchhash type=%d size=%zd Bval=%d\n", data.type, datasize, (int)data.value.B));
   /* Create a string SV of apropriate size.  The string is arbitrary
    * and just has to be larger than the largest datatype.   */
-  ret->datasv = newSVpvn("                                ",pdl_howbig(ret->datatype = data.type));
+  ret->datasv = newSVpvn("                                ",datasize);
   ret->data = SvPV((SV *)ret->datasv,n_a);
   /* Refcnt should be 1 already... */
   /* Make the whole pdl mortal so destruction happens at the right time.
@@ -71,6 +73,7 @@ pdl* pdl_SvPDLV ( SV* sv ) {
       /* The scalar is not a ref, so we can use direct conversion. */
       PDL_Anyval data;
       ANYVAL_FROM_SV(data, sv, TRUE, -1);
+      PDLDEBUG_f(printf("pdl_SvPDLV type: %d\n", data.type));
       return pdl_scalar(data);
    } /* End of scalar case */
 
@@ -243,28 +246,27 @@ static void pdl_barf_or_warn(const char* pat, int iswarn, va_list* args)
      * the perl-level PDL::barf() or PDL::cluck()
      */
 
-    { /* scope block for C89 compatibility */
-
-        SV * sv;
-
-        dSP;
-        ENTER;
-        SAVETMPS;
-        PUSHMARK(SP);
-
-        sv = sv_2mortal(newSV(0));
-        sv_vsetpvfn(sv, pat, strlen(pat), args, Null(SV**), 0, Null(bool*));
-        va_end(*args);
-
-        XPUSHs(sv);
-
-        PUTBACK;
-
-        call_pv(iswarn ? "PDL::cluck" : "PDL::barf", G_DISCARD);
-
-        FREETMPS;
-        LEAVE;
-    } /* end C89 compatibility scope block */
+    dSP;
+    ENTER;
+    SAVETMPS;
+    PUSHMARK(SP);
+    SV *sv = sv_2mortal(newSV(0));
+    int size = vsnprintf(NULL, 0, pat, *args);
+    va_end(*args);
+    if (size < 0) {
+      sv_setpv(sv, "vsnprintf error");
+    } else {
+      size += 2;             /* For '\0' + 1 as CentOS 7 is off by 1 */
+      char buf[size];
+      size = vsnprintf(buf, size, pat, *args);
+      va_end(*args);
+      sv_setpv(sv, size < 0 ? "vsnprintf error" : buf);
+    }
+    XPUSHs(sv);
+    PUTBACK;
+    call_pv(iswarn ? "PDL::cluck" : "PDL::barf", G_DISCARD);
+    FREETMPS;
+    LEAVE;
 }
 
 #define GEN_PDL_BARF_OR_WARN_I_STDARG(type, iswarn)     \
@@ -488,7 +490,7 @@ static int _detect_datatype(AV *av) {
 pdl* pdl_from_array(AV* av, AV* dims, int dtype, pdl* dest_pdl)
 {
   int ndims, i, level=0;
-  PDL_Anyval undefval = { -1, {0} };
+  PDL_Anyval undefval = { PDL_INVALID, {0} };
 
   ndims = av_len(dims)+1;
   PDL_Indx dest_dims[ndims];
@@ -513,10 +515,11 @@ pdl* pdl_from_array(AV* av, AV* dims, int dtype, pdl* dest_pdl)
   /******
    * Copy the undefval to fill empty spots in the ndarray...
    */
+  PDLDEBUG_f(printf("pdl_from_array type: %d\n", dtype));
   ANYVAL_FROM_SV(undefval, NULL, TRUE, dtype);
 #define X(dtype, ctype, ppsym, shortctype, defbval) \
     pdl_setav_ ## ppsym(dest_pdl->data,av,dest_dims,ndims,level, undefval.value.ppsym, dest_pdl);
-  PDL_GENERICSWITCH(dtype, X, return NULL)
+  PDL_GENERICSWITCH(PDL_TYPELIST2_ALL, dtype, X, return NULL)
 #undef X
   return dest_pdl;
 }
@@ -538,7 +541,7 @@ PDL_Indx pdl_get_offset(PDL_Indx* pos, PDL_Indx* dims, PDL_Indx *incs, PDL_Indx
 
 /* wrapper for pdl_at where only want first item, cf sclr_c */
 PDL_Anyval pdl_at0( pdl* it ) {
-    PDL_Anyval result = { -1, {0} };
+    PDL_Anyval result = { PDL_INVALID, {0} };
     PDL_Indx nullp = 0;
     PDL_Indx dummyd = 1;
     PDL_Indx dummyi = 1;
@@ -552,7 +555,7 @@ PDL_Anyval pdl_at0( pdl* it ) {
 /* Return value at position (x,y,z...) */
 PDL_Anyval pdl_at( void* x, int datatype, PDL_Indx* pos, PDL_Indx* dims,
 	PDL_Indx* incs, PDL_Indx offset, PDL_Indx ndims) {
-   PDL_Anyval result = { -1, {0} };
+   PDL_Anyval result = { PDL_INVALID, {0} };
    PDL_Indx ioff = pdl_get_offset(pos, dims, incs, offset, ndims);
    if (ioff < 0) return result;
    ANYVAL_FROM_CTYPE_OFFSET(result, datatype, x, ioff);
@@ -673,9 +676,11 @@ PDL_Indx pdl_kludge_copy_ ## ppsym_out(PDL_Indx dest_off, /* Offset into the des
     } \
     /* This is used inside the switch in order to detect badvalues. */ \
     PDL_Anyval source_badval = pdl_get_pdl_badvalue(source_pdl); \
+    if (source_badval.type < 0) barf("Error getting badvalue, type=%d", source_badval.type); \
     PDL_Anyval dest_badval = pdl_get_pdl_badvalue(dest_pdl); \
+    if (dest_badval.type < 0) barf("Error getting badvalue, type=%d", dest_badval.type); \
     char found_bad = 0; \
-    PDL_GENERICSWITCH2(source_pdl->datatype, X, croak("Not a known data type code=%d", source_pdl->datatype)) \
+    PDL_GENERICSWITCH(PDL_TYPELIST2_ALL_, source_pdl->datatype, X, croak("Not a known data type code=%d", source_pdl->datatype)) \
     return undef_count; \
   } \
   /* If we are here, we are not at the bottom level yet.  So walk \
@@ -720,7 +725,7 @@ PDL_Indx pdl_kludge_copy_ ## ppsym_out(PDL_Indx dest_off, /* Offset into the des
   } /* end of padding IF statement */ \
   return undef_count; \
 }
-PDL_GENERICLIST2(PDL_KLUDGE_COPY_X, INNERLOOP_X)
+PDL_TYPELIST2_ALL(PDL_KLUDGE_COPY_X, INNERLOOP_X)
 #undef PDL_KLUDGE_COPY_X
 
 /*
@@ -812,7 +817,7 @@ PDL_Indx pdl_setav_ ## ppsym_out(ctype_out* dest_data, AV* av, \
   return undef_count; \
 }
 
-PDL_GENERICLIST2(PDL_SETAV_X, INNERLOOP_X)
+PDL_TYPELIST2_ALL(PDL_SETAV_X, INNERLOOP_X)
 #undef PDL_SETAV_X
 #undef INNERLOOP_X
 
