@@ -22,15 +22,16 @@ PDL::Graphics::TriD::Contours - 3D Surface contours for TriD
 
 package PDL::Graphics::TriD::Contours;
 use strict;
+use warnings;
 use PDL;
 use PDL::Graphics::TriD;
 use PDL::Graphics::TriD::Rout;
 use PDL::Graphics::TriD::Labels;
-#use Data::Dumper;
-
 use base qw/PDL::Graphics::TriD::GObject/;
 use fields qw/ContourSegCnt Labels LabelStrings/;
 
+$PDL::Graphics::TriD::verbose //= 0;
+
 =head1 FUNCTIONS
 
 =head2 new()
@@ -206,7 +207,6 @@ sub addlabels{
   my $lp=pdl->null;
 
   my $pcnt = 0;
-  my $cnt;
   my $offset = pdl[0.5,0.5,0.5];
 
   for(my $i=0; $i<= $#{$self->{ContourSegCnt}}; $i++){
